@@ -15,6 +15,7 @@ import (
 	"syscall"
 	"time"
 
+	"github.com/containers/common/libnetwork/slirp4netns"
 	"github.com/containers/common/libnetwork/types"
 	"github.com/containers/common/pkg/cgroups"
 	"github.com/containers/common/pkg/config"
@@ -29,7 +30,7 @@ import (
 )
 
 var (
-	bindOptions = []string{"bind", "rprivate"}
+	bindOptions = []string{define.TypeBind, "rprivate"}
 )
 
 func (c *Container) mountSHM(shmOptions string) error {
@@ -38,7 +39,7 @@ func (c *Container) mountSHM(shmOptions string) error {
 		contextType = "rootcontext"
 	}
 
-	if err := unix.Mount("shm", c.config.ShmDir, "tmpfs", unix.MS_NOEXEC|unix.MS_NOSUID|unix.MS_NODEV,
+	if err := unix.Mount("shm", c.config.ShmDir, define.TypeTmpfs, unix.MS_NOEXEC|unix.MS_NOSUID|unix.MS_NODEV,
 		label.FormatMountLabelByType(shmOptions, c.config.MountLabel, contextType)); err != nil {
 		return fmt.Errorf("failed to mount shm tmpfs %q: %w", c.config.ShmDir, err)
 	}
@@ -224,8 +225,8 @@ func (c *Container) setupSystemd(mounts []spec.Mount, g generate.Generator) erro
 		}
 		tmpfsMnt := spec.Mount{
 			Destination: dest,
-			Type:        "tmpfs",
-			Source:      "tmpfs",
+			Type:        define.TypeTmpfs,
+			Source:      define.TypeTmpfs,
 			Options:     append(options, "tmpcopyup", shmSizeSystemdMntOpt),
 		}
 		g.AddMount(tmpfsMnt)
@@ -236,8 +237,8 @@ func (c *Container) setupSystemd(mounts []spec.Mount, g generate.Generator) erro
 		}
 		tmpfsMnt := spec.Mount{
 			Destination: dest,
-			Type:        "tmpfs",
-			Source:      "tmpfs",
+			Type:        define.TypeTmpfs,
+			Source:      define.TypeTmpfs,
 			Options:     append(options, "tmpcopyup", shmSizeSystemdMntOpt),
 		}
 		g.AddMount(tmpfsMnt)
@@ -270,9 +271,9 @@ func (c *Container) setupSystemd(mounts []spec.Mount, g generate.Generator) erro
 		} else {
 			systemdMnt = spec.Mount{
 				Destination: "/sys/fs/cgroup",
-				Type:        "bind",
+				Type:        define.TypeBind,
 				Source:      "/sys/fs/cgroup",
-				Options:     []string{"bind", "private", "rw"},
+				Options:     []string{define.TypeBind, "private", "rw"},
 			}
 		}
 		g.AddMount(systemdMnt)
@@ -281,7 +282,7 @@ func (c *Container) setupSystemd(mounts []spec.Mount, g generate.Generator) erro
 		if hasCgroupNs && !hasSystemdMount {
 			return errors.New("cgroup namespace is not supported with cgroup v1 and systemd mode")
 		}
-		mountOptions := []string{"bind", "rprivate"}
+		mountOptions := []string{define.TypeBind, "rprivate"}
 
 		if !hasSystemdMount {
 			skipMount := hasSystemdMount
@@ -310,7 +311,7 @@ func (c *Container) setupSystemd(mounts []spec.Mount, g generate.Generator) erro
 			if !skipMount {
 				systemdMnt := spec.Mount{
 					Destination: "/sys/fs/cgroup/systemd",
-					Type:        "bind",
+					Type:        define.TypeBind,
 					Source:      "/sys/fs/cgroup/systemd",
 					Options:     mountOptions,
 				}
@@ -621,7 +622,7 @@ func (c *Container) setCgroupsPath(g *generate.Generator) error {
 func (c *Container) addSlirp4netnsDNS(nameservers []string) []string {
 	// slirp4netns has a built in DNS forwarder.
 	if c.config.NetMode.IsSlirp4netns() {
-		slirp4netnsDNS, err := GetSlirp4netnsDNS(c.slirp4netnsSubnet)
+		slirp4netnsDNS, err := slirp4netns.GetDNS(c.slirp4netnsSubnet)
 		if err != nil {
 			logrus.Warn("Failed to determine Slirp4netns DNS: ", err.Error())
 		} else {
@@ -631,20 +632,28 @@ func (c *Container) addSlirp4netnsDNS(nameservers []string) []string {
 	return nameservers
 }
 
-func (c *Container) isSlirp4netnsIPv6() (bool, error) {
+func (c *Container) isSlirp4netnsIPv6() bool {
 	if c.config.NetMode.IsSlirp4netns() {
-		ctrNetworkSlipOpts := []string{}
-		if c.config.NetworkOptions != nil {
-			ctrNetworkSlipOpts = append(ctrNetworkSlipOpts, c.config.NetworkOptions["slirp4netns"]...)
+		extraOptions := c.config.NetworkOptions[slirp4netns.BinaryName]
+		options := make([]string, 0, len(c.runtime.config.Engine.NetworkCmdOptions)+len(extraOptions))
+		options = append(options, c.runtime.config.Engine.NetworkCmdOptions...)
+		options = append(options, extraOptions...)
+
+		// loop backwards as the last argument wins and we can exit early
+		// This should be kept in sync with c/common/libnetwork/slirp4netns.
+		for i := len(options) - 1; i >= 0; i-- {
+			switch options[i] {
+			case "enable_ipv6=true":
+				return true
+			case "enable_ipv6=false":
+				return false
+			}
 		}
-		slirpOpts, err := parseSlirp4netnsNetworkOptions(c.runtime, ctrNetworkSlipOpts)
-		if err != nil {
-			return false, err
-		}
-		return slirpOpts.enableIPv6, nil
+		// default is true
+		return true
 	}
 
-	return false, nil
+	return false
 }
 
 // check for net=none
@@ -750,7 +759,7 @@ func (c *Container) safeMountSubPath(mountPoint, subpath string) (s *safeMountIn
 	if err != nil {
 		return nil, err
 	}
-	npath := ""
+	var npath string
 	switch {
 	case fi.Mode()&fs.ModeSymlink != 0:
 		return nil, fmt.Errorf("file %q is a symlink", joinedPath)
@@ -775,3 +784,20 @@ func (c *Container) safeMountSubPath(mountPoint, subpath string) (s *safeMountIn
 		mountPoint: npath,
 	}, nil
 }
+
+func (c *Container) makePlatformMtabLink(etcInTheContainerFd, rootUID, rootGID int) error {
+	// If /etc/mtab does not exist in container image, then we need to
+	// create it, so that mount command within the container will work.
+	err := unix.Symlinkat("/proc/mounts", etcInTheContainerFd, "mtab")
+	if err != nil && !os.IsExist(err) {
+		return fmt.Errorf("creating /etc/mtab symlink: %w", err)
+	}
+	// If the symlink was created, then also chown it to root in the container
+	if err == nil && (rootUID != 0 || rootGID != 0) {
+		err = unix.Fchownat(etcInTheContainerFd, "mtab", rootUID, rootGID, unix.AT_SYMLINK_NOFOLLOW)
+		if err != nil {
+			return fmt.Errorf("chown /etc/mtab: %w", err)
+		}
+	}
+	return nil
+}
