@@ -249,16 +249,7 @@ func (t *task) report() *entities.AutoUpdateReport {
 func (t *task) updateAvailable(ctx context.Context) (bool, error) {
 	switch t.policy {
 	case PolicyRegistryImage:
-		// Errors checking for updates only should not be fatal.
-		// Especially on Edge systems, connection may be limited or
-		// there may just be a temporary downtime of the registry.
-		// But make sure to leave some breadcrumbs in the debug logs
-		// such that potential issues _can_ be analyzed if needed.
-		available, err := t.registryUpdateAvailable(ctx)
-		if err != nil {
-			logrus.Debugf("Error checking updates for image %s: %v (ignoring error)", t.rawImageName, err)
-		}
-		return available, nil
+		return t.registryUpdateAvailable(ctx)
 	case PolicyLocalImage:
 		return t.localUpdateAvailable()
 	default:
@@ -291,7 +282,10 @@ func (t *task) registryUpdateAvailable(ctx context.Context) (bool, error) {
 	if err != nil {
 		return false, err
 	}
-	options := &libimage.HasDifferentDigestOptions{AuthFilePath: t.authfile}
+	options := &libimage.HasDifferentDigestOptions{
+		AuthFilePath:          t.authfile,
+		InsecureSkipTLSVerify: t.auto.options.InsecureSkipTLSVerify,
+	}
 	return t.image.HasDifferentDigest(ctx, remoteRef, options)
 }
 
@@ -305,6 +299,7 @@ func (t *task) registryUpdate(ctx context.Context) error {
 	pullOptions := &libimage.PullOptions{}
 	pullOptions.AuthFilePath = t.authfile
 	pullOptions.Writer = os.Stderr
+	pullOptions.InsecureSkipTLSVerify = t.auto.options.InsecureSkipTLSVerify
 	if _, err := t.auto.runtime.LibimageRuntime().Pull(ctx, t.rawImageName, config.PullPolicyAlways, pullOptions); err != nil {
 		return err
 	}
@@ -334,16 +329,8 @@ func (t *task) rollbackImage() error {
 
 // restartSystemdUnit restarts the systemd unit the container is running in.
 func (u *updater) restartSystemdUnit(ctx context.Context, unit string) error {
-	if err := u.stopSystemdUnit(ctx, unit); err != nil {
-		return err
-	}
-	return u.startSystemdUnit(ctx, unit)
-}
-
-// startSystemdUnit starts the systemd unit the container is running in.
-func (u *updater) startSystemdUnit(ctx context.Context, unit string) error {
 	restartChan := make(chan string)
-	if _, err := u.conn.StartUnitContext(ctx, unit, "replace", restartChan); err != nil {
+	if _, err := u.conn.RestartUnitContext(ctx, unit, "replace", restartChan); err != nil {
 		return err
 	}
 
@@ -357,28 +344,7 @@ func (u *updater) startSystemdUnit(ctx context.Context, unit string) error {
 		return nil
 
 	default:
-		return fmt.Errorf("error starting systemd unit %q expected %q but received %q", unit, "done", result)
-	}
-}
-
-// stopSystemdUnit stop the systemd unit the container is running in.
-func (u *updater) stopSystemdUnit(ctx context.Context, unit string) error {
-	restartChan := make(chan string)
-	if _, err := u.conn.StopUnitContext(ctx, unit, "replace", restartChan); err != nil {
-		return err
-	}
-
-	// Wait for the restart to finish and actually check if it was
-	// successful or not.
-	result := <-restartChan
-
-	switch result {
-	case "done":
-		logrus.Infof("Successfully stopped systemd unit %q", unit)
-		return nil
-
-	default:
-		return fmt.Errorf("error stopping systemd unit %q expected %q but received %q", unit, "done", result)
+		return fmt.Errorf("error restarting systemd unit %q expected %q but received %q", unit, "done", result)
 	}
 }
 
@@ -454,8 +420,14 @@ func (u *updater) assembleTasks(ctx context.Context) []error {
 			continue
 		}
 
+		// Use user-specified auth file (CLI or env variable) unless
+		// the container was created with the auth-file label.
+		authfile := u.options.Authfile
+		if fromContainer, ok := labels[define.AutoUpdateAuthfileLabel]; ok {
+			authfile = fromContainer
+		}
 		t := task{
-			authfile:     labels[define.AutoUpdateAuthfileLabel],
+			authfile:     authfile,
 			auto:         u,
 			container:    ctr,
 			policy:       policy,
