@@ -1,30 +1,35 @@
+//go:build linux && cgo
 // +build linux,cgo
 
 package rootless
 
 import (
 	"bufio"
+	"bytes"
+	"errors"
 	"fmt"
 	"io"
-	"io/ioutil"
 	"os"
 	"os/exec"
 	gosignal "os/signal"
 	"os/user"
 	"runtime"
 	"strconv"
+	"strings"
 	"sync"
 	"unsafe"
 
-	"github.com/containers/podman/v2/pkg/errorhandling"
+	"github.com/containers/podman/v4/pkg/errorhandling"
 	"github.com/containers/storage/pkg/idtools"
-	"github.com/pkg/errors"
+	pmount "github.com/containers/storage/pkg/mount"
+	"github.com/containers/storage/pkg/unshare"
 	"github.com/sirupsen/logrus"
+	"github.com/syndtr/gocapability/capability"
 	"golang.org/x/sys/unix"
 )
 
 /*
-#cgo remoteclient CFLAGS: -Wall -Werror -DDISABLE_JOIN_SHORTCUT
+#cgo remote CFLAGS: -Wall -Werror -DDISABLE_JOIN_SHORTCUT
 #include <stdlib.h>
 #include <sys/types.h>
 extern uid_t rootless_uid();
@@ -57,16 +62,25 @@ func IsRootless() bool {
 		if rootlessUIDInit != 0 {
 			// This happens if we joined the user+mount namespace as part of
 			if err := os.Setenv("_CONTAINERS_USERNS_CONFIGURED", "done"); err != nil {
-				logrus.Errorf("failed to set environment variable %s as %s", "_CONTAINERS_USERNS_CONFIGURED", "done")
+				logrus.Errorf("Failed to set environment variable %s as %s", "_CONTAINERS_USERNS_CONFIGURED", "done")
 			}
 			if err := os.Setenv("_CONTAINERS_ROOTLESS_UID", fmt.Sprintf("%d", rootlessUIDInit)); err != nil {
-				logrus.Errorf("failed to set environment variable %s as %d", "_CONTAINERS_ROOTLESS_UID", rootlessUIDInit)
+				logrus.Errorf("Failed to set environment variable %s as %d", "_CONTAINERS_ROOTLESS_UID", rootlessUIDInit)
 			}
 			if err := os.Setenv("_CONTAINERS_ROOTLESS_GID", fmt.Sprintf("%d", rootlessGIDInit)); err != nil {
-				logrus.Errorf("failed to set environment variable %s as %d", "_CONTAINERS_ROOTLESS_GID", rootlessGIDInit)
+				logrus.Errorf("Failed to set environment variable %s as %d", "_CONTAINERS_ROOTLESS_GID", rootlessGIDInit)
 			}
 		}
 		isRootless = os.Geteuid() != 0 || os.Getenv("_CONTAINERS_USERNS_CONFIGURED") != ""
+		if !isRootless {
+			hasCapSysAdmin, err := unshare.HasCapSysAdmin()
+			if err != nil {
+				logrus.Warnf("Failed to read CAP_SYS_ADMIN presence for the current process")
+			}
+			if err == nil && !hasCapSysAdmin {
+				isRootless = true
+			}
+		}
 	})
 	return isRootless
 }
@@ -100,12 +114,18 @@ func GetRootlessGID() int {
 
 func tryMappingTool(uid bool, pid int, hostID int, mappings []idtools.IDMap) error {
 	var tool = "newuidmap"
+	mode := os.ModeSetuid
+	cap := capability.CAP_SETUID
+	idtype := "setuid"
 	if !uid {
 		tool = "newgidmap"
+		mode = os.ModeSetgid
+		cap = capability.CAP_SETGID
+		idtype = "setgid"
 	}
 	path, err := exec.LookPath(tool)
 	if err != nil {
-		return errors.Wrapf(err, "cannot find %s", tool)
+		return fmt.Errorf("command required for rootless mode with multiple IDs: %w", err)
 	}
 
 	appendTriplet := func(l []string, a, b, c int) []string {
@@ -122,7 +142,7 @@ func tryMappingTool(uid bool, pid int, hostID int, mappings []idtools.IDMap) err
 				what = "GID"
 				where = "/etc/subgid"
 			}
-			return errors.Errorf("invalid configuration: the specified mapping %d:%d in %q includes the user %s", i.HostID, i.Size, where, what)
+			return fmt.Errorf("invalid configuration: the specified mapping %d:%d in %q includes the user %s", i.HostID, i.Size, where, what)
 		}
 		args = appendTriplet(args, i.ContainerID+1, i.HostID, i.Size)
 	}
@@ -132,8 +152,14 @@ func tryMappingTool(uid bool, pid int, hostID int, mappings []idtools.IDMap) err
 	}
 
 	if output, err := cmd.CombinedOutput(); err != nil {
-		logrus.Debugf("error from %s: %s", tool, output)
-		return errors.Wrapf(err, "cannot setup namespace using %s", tool)
+		logrus.Errorf("running `%s`: %s", strings.Join(args, " "), output)
+		errorStr := fmt.Sprintf("cannot set up namespace using %q", path)
+		if isSet, err := unshare.IsSetID(cmd.Path, mode, cap); err != nil {
+			logrus.Errorf("Failed to check for %s on %s: %v", idtype, path, err)
+		} else if !isSet {
+			errorStr = fmt.Sprintf("%s: should have %s or have filecaps %s", errorStr, idtype, idtype)
+		}
+		return fmt.Errorf("%v: %w", errorStr, err)
 	}
 	return nil
 }
@@ -142,8 +168,12 @@ func tryMappingTool(uid bool, pid int, hostID int, mappings []idtools.IDMap) err
 // namespace of the specified PID without looking up its parent.  Useful to join directly
 // the conmon process.
 func joinUserAndMountNS(pid uint, pausePid string) (bool, int, error) {
-	if os.Geteuid() == 0 || os.Getenv("_CONTAINERS_USERNS_CONFIGURED") != "" {
-		return false, -1, nil
+	hasCapSysAdmin, err := unshare.HasCapSysAdmin()
+	if err != nil {
+		return false, 0, err
+	}
+	if (os.Geteuid() == 0 && hasCapSysAdmin) || os.Getenv("_CONTAINERS_USERNS_CONFIGURED") != "" {
+		return false, 0, nil
 	}
 
 	cPausePid := C.CString(pausePid)
@@ -151,19 +181,14 @@ func joinUserAndMountNS(pid uint, pausePid string) (bool, int, error) {
 
 	pidC := C.reexec_userns_join(C.int(pid), cPausePid)
 	if int(pidC) < 0 {
-		return false, -1, errors.Errorf("cannot re-exec process")
+		return false, -1, fmt.Errorf("cannot re-exec process to join the existing user namespace")
 	}
 
-	ret := C.reexec_in_user_namespace_wait(pidC, 0)
-	if ret < 0 {
-		return false, -1, errors.New("error waiting for the re-exec process")
-	}
-
-	return true, int(ret), nil
+	return waitAndProxySignalsToChild(pidC)
 }
 
 // GetConfiguredMappings returns the additional IDs configured for the current user.
-func GetConfiguredMappings() ([]idtools.IDMap, []idtools.IDMap, error) {
+func GetConfiguredMappings(quiet bool) ([]idtools.IDMap, []idtools.IDMap, error) {
 	var uids, gids []idtools.IDMap
 	username := os.Getenv("USER")
 	if username == "" {
@@ -180,8 +205,11 @@ func GetConfiguredMappings() ([]idtools.IDMap, []idtools.IDMap, error) {
 	}
 	mappings, err := idtools.NewIDMappings(username, username)
 	if err != nil {
-		logrus.Errorf(
-			"cannot find UID/GID for user %s: %v - check rootless mode in man pages.", username, err)
+		logLevel := logrus.ErrorLevel
+		if quiet || (os.Geteuid() == 0 && GetRootlessUID() == 0) {
+			logLevel = logrus.DebugLevel
+		}
+		logrus.StandardLogger().Logf(logLevel, "cannot find UID/GID for user %s: %v - check rootless mode in man pages.", username, err)
 	} else {
 		uids = mappings.UIDs()
 		gids = mappings.GIDs()
@@ -189,14 +217,59 @@ func GetConfiguredMappings() ([]idtools.IDMap, []idtools.IDMap, error) {
 	return uids, gids, nil
 }
 
+func copyMappings(from, to string) error {
+	// when running as non-root always go through the newuidmap/newgidmap
+	// configuration since this is the expectation when running on Kubernetes
+	if os.Geteuid() != 0 {
+		return errors.New("copying mappings is allowed only for root")
+	}
+	content, err := os.ReadFile(from)
+	if err != nil {
+		return err
+	}
+	// Both runc and crun check whether the current process is in a user namespace
+	// by looking up 4294967295 in /proc/self/uid_map.  If the mappings would be
+	// copied as they are, the check in the OCI runtimes would fail.  So just split
+	// it in two different ranges.
+	if bytes.Contains(content, []byte("4294967295")) {
+		content = []byte("0 0 1\n1 1 4294967294\n")
+	}
+	return os.WriteFile(to, content, 0600)
+}
+
 func becomeRootInUserNS(pausePid, fileToRead string, fileOutput *os.File) (_ bool, _ int, retErr error) {
-	if os.Geteuid() == 0 || os.Getenv("_CONTAINERS_USERNS_CONFIGURED") != "" {
+	hasCapSysAdmin, err := unshare.HasCapSysAdmin()
+	if err != nil {
+		return false, 0, err
+	}
+
+	if (os.Geteuid() == 0 && hasCapSysAdmin) || os.Getenv("_CONTAINERS_USERNS_CONFIGURED") != "" {
 		if os.Getenv("_CONTAINERS_USERNS_CONFIGURED") == "init" {
 			return false, 0, runInUser()
 		}
 		return false, 0, nil
 	}
 
+	if _, inContainer := os.LookupEnv("container"); !inContainer {
+		if mounts, err := pmount.GetMounts(); err == nil {
+			for _, m := range mounts {
+				if m.Mountpoint == "/" {
+					isShared := false
+					for _, o := range strings.Split(m.Optional, ",") {
+						if strings.HasPrefix(o, "shared:") {
+							isShared = true
+							break
+						}
+					}
+					if !isShared {
+						logrus.Warningf("%q is not a shared mount, this could cause issues or missing mounts with rootless containers", m.Mountpoint)
+					}
+					break
+				}
+			}
+		}
+	}
+
 	cPausePid := C.CString(pausePid)
 	defer C.free(unsafe.Pointer(cPausePid))
 
@@ -226,11 +299,13 @@ func becomeRootInUserNS(pausePid, fileToRead string, fileOutput *os.File) (_ boo
 			toWrite = []byte("1")
 		}
 		if _, err := w.Write(toWrite); err != nil {
-			logrus.Errorf("failed to write byte 0: %q", err)
+			logrus.Errorf("Failed to write byte 0: %q", err)
 		}
 		if retErr != nil && pid > 0 {
 			if err := unix.Kill(pid, unix.SIGKILL); err != nil {
-				logrus.Errorf("failed to kill %d", pid)
+				if err != unix.ESRCH {
+					logrus.Errorf("Failed to clean up process %d: %v", pid, err)
+				}
 			}
 			C.reexec_in_user_namespace_wait(C.int(pid), 0)
 		}
@@ -239,16 +314,24 @@ func becomeRootInUserNS(pausePid, fileToRead string, fileOutput *os.File) (_ boo
 	pidC := C.reexec_in_user_namespace(C.int(r.Fd()), cPausePid, cFileToRead, fileOutputFD)
 	pid = int(pidC)
 	if pid < 0 {
-		return false, -1, errors.Errorf("cannot re-exec process")
+		return false, -1, fmt.Errorf("cannot re-exec process")
 	}
 
-	uids, gids, err := GetConfiguredMappings()
+	uids, gids, err := GetConfiguredMappings(false)
 	if err != nil {
 		return false, -1, err
 	}
 
+	uidMap := fmt.Sprintf("/proc/%d/uid_map", pid)
+	gidMap := fmt.Sprintf("/proc/%d/gid_map", pid)
+
 	uidsMapped := false
-	if uids != nil {
+
+	if err := copyMappings("/proc/self/uid_map", uidMap); err == nil {
+		uidsMapped = true
+	}
+
+	if uids != nil && !uidsMapped {
 		err := tryMappingTool(true, pid, os.Geteuid(), uids)
 		// If some mappings were specified, do not ignore the error
 		if err != nil && len(uids) > 0 {
@@ -257,24 +340,26 @@ func becomeRootInUserNS(pausePid, fileToRead string, fileOutput *os.File) (_ boo
 		uidsMapped = err == nil
 	}
 	if !uidsMapped {
-		logrus.Warnf("using rootless single mapping into the namespace. This might break some images. Check /etc/subuid and /etc/subgid for adding subids")
+		logrus.Warnf("Using rootless single mapping into the namespace. This might break some images. Check /etc/subuid and /etc/subgid for adding sub*ids if not using a network user")
 		setgroups := fmt.Sprintf("/proc/%d/setgroups", pid)
-		err = ioutil.WriteFile(setgroups, []byte("deny\n"), 0666)
+		err = os.WriteFile(setgroups, []byte("deny\n"), 0666)
 		if err != nil {
-			return false, -1, errors.Wrapf(err, "cannot write setgroups file")
+			return false, -1, fmt.Errorf("cannot write setgroups file: %w", err)
 		}
 		logrus.Debugf("write setgroups file exited with 0")
 
-		uidMap := fmt.Sprintf("/proc/%d/uid_map", pid)
-		err = ioutil.WriteFile(uidMap, []byte(fmt.Sprintf("%d %d 1\n", 0, os.Geteuid())), 0666)
+		err = os.WriteFile(uidMap, []byte(fmt.Sprintf("%d %d 1\n", 0, os.Geteuid())), 0666)
 		if err != nil {
-			return false, -1, errors.Wrapf(err, "cannot write uid_map")
+			return false, -1, fmt.Errorf("cannot write uid_map: %w", err)
 		}
 		logrus.Debugf("write uid_map exited with 0")
 	}
 
 	gidsMapped := false
-	if gids != nil {
+	if err := copyMappings("/proc/self/gid_map", gidMap); err == nil {
+		gidsMapped = true
+	}
+	if gids != nil && !gidsMapped {
 		err := tryMappingTool(false, pid, os.Getegid(), gids)
 		// If some mappings were specified, do not ignore the error
 		if err != nil && len(gids) > 0 {
@@ -283,30 +368,28 @@ func becomeRootInUserNS(pausePid, fileToRead string, fileOutput *os.File) (_ boo
 		gidsMapped = err == nil
 	}
 	if !gidsMapped {
-		gidMap := fmt.Sprintf("/proc/%d/gid_map", pid)
-		err = ioutil.WriteFile(gidMap, []byte(fmt.Sprintf("%d %d 1\n", 0, os.Getegid())), 0666)
+		err = os.WriteFile(gidMap, []byte(fmt.Sprintf("%d %d 1\n", 0, os.Getegid())), 0666)
 		if err != nil {
-			return false, -1, errors.Wrapf(err, "cannot write gid_map")
+			return false, -1, fmt.Errorf("cannot write gid_map: %w", err)
 		}
 	}
 
-	_, err = w.Write([]byte("0"))
+	_, err = w.WriteString("0")
 	if err != nil {
-		return false, -1, errors.Wrapf(err, "write to sync pipe")
+		return false, -1, fmt.Errorf("write to sync pipe: %w", err)
 	}
 
 	b := make([]byte, 1)
 	_, err = w.Read(b)
 	if err != nil {
-		return false, -1, errors.Wrapf(err, "read from sync pipe")
+		return false, -1, fmt.Errorf("read from sync pipe: %w", err)
 	}
 
 	if fileOutput != nil {
 		ret := C.reexec_in_user_namespace_wait(pidC, 0)
 		if ret < 0 {
-			return false, -1, errors.New("error waiting for the re-exec process")
+			return false, -1, errors.New("waiting for the re-exec process")
 		}
-
 		return true, 0, nil
 	}
 
@@ -314,22 +397,25 @@ func becomeRootInUserNS(pausePid, fileToRead string, fileOutput *os.File) (_ boo
 		// We have lost the race for writing the PID file, as probably another
 		// process created a namespace and wrote the PID.
 		// Try to join it.
-		data, err := ioutil.ReadFile(pausePid)
+		data, err := os.ReadFile(pausePid)
 		if err == nil {
-			pid, err := strconv.ParseUint(string(data), 10, 0)
+			var pid uint64
+			pid, err = strconv.ParseUint(string(data), 10, 0)
 			if err == nil {
 				return joinUserAndMountNS(uint(pid), "")
 			}
 		}
-		return false, -1, errors.Wrapf(err, "error setting up the process")
+		return false, -1, fmt.Errorf("setting up the process: %w", err)
 	}
 
 	if b[0] != '0' {
-		return false, -1, errors.Wrapf(err, "error setting up the process")
+		return false, -1, errors.New("setting up the process")
 	}
 
-	c := make(chan os.Signal, 1)
+	return waitAndProxySignalsToChild(pidC)
+}
 
+func waitAndProxySignalsToChild(pid C.int) (bool, int, error) {
 	signals := []os.Signal{}
 	for sig := 0; sig < numSig; sig++ {
 		if sig == int(unix.SIGTSTP) {
@@ -338,23 +424,30 @@ func becomeRootInUserNS(pausePid, fileToRead string, fileOutput *os.File) (_ boo
 		signals = append(signals, unix.Signal(sig))
 	}
 
+	// Disable all existing signal handlers, from now forward everything to the child and let
+	// it deal with it. All we do is to wait and propagate the exit code from the child to our parent.
+	gosignal.Reset()
+	c := make(chan os.Signal, len(signals))
 	gosignal.Notify(c, signals...)
-	defer gosignal.Reset()
 	go func() {
 		for s := range c {
 			if s == unix.SIGCHLD || s == unix.SIGPIPE {
 				continue
 			}
 
-			if err := unix.Kill(int(pidC), s.(unix.Signal)); err != nil {
-				logrus.Errorf("failed to kill %d", int(pidC))
+			if err := unix.Kill(int(pid), s.(unix.Signal)); err != nil {
+				if err != unix.ESRCH {
+					logrus.Errorf("Failed to propagate signal to child process %d: %v", int(pid), err)
+				}
 			}
 		}
 	}()
 
-	ret := C.reexec_in_user_namespace_wait(pidC, 0)
+	ret := C.reexec_in_user_namespace_wait(pid, 0)
+	// child exited reset our signal proxy handler
+	gosignal.Reset()
 	if ret < 0 {
-		return false, -1, errors.New("error waiting for the re-exec process")
+		return false, -1, errors.New("waiting for the re-exec process")
 	}
 
 	return true, int(ret), nil
@@ -377,17 +470,12 @@ func BecomeRootInUserNS(pausePid string) (bool, int, error) {
 // different uidmap and the unprivileged user has no way to read the
 // file owned by the root in the container.
 func TryJoinFromFilePaths(pausePidPath string, needNewNamespace bool, paths []string) (bool, int, error) {
-	if len(paths) == 0 {
-		return BecomeRootInUserNS(pausePidPath)
-	}
-
 	var lastErr error
 	var pausePid int
-	foundProcess := false
 
 	for _, path := range paths {
 		if !needNewNamespace {
-			data, err := ioutil.ReadFile(path)
+			data, err := os.ReadFile(path)
 			if err != nil {
 				lastErr = err
 				continue
@@ -395,12 +483,9 @@ func TryJoinFromFilePaths(pausePidPath string, needNewNamespace bool, paths []st
 
 			pausePid, err = strconv.Atoi(string(data))
 			if err != nil {
-				lastErr = errors.Wrapf(err, "cannot parse file %s", path)
+				lastErr = fmt.Errorf("cannot parse file %q: %w", path, err)
 				continue
 			}
-
-			lastErr = nil
-			break
 		} else {
 			r, w, err := os.Pipe()
 			if err != nil {
@@ -427,26 +512,29 @@ func TryJoinFromFilePaths(pausePidPath string, needNewNamespace bool, paths []st
 
 			n, err := r.Read(b)
 			if err != nil {
-				lastErr = errors.Wrapf(err, "cannot read %s\n", path)
+				lastErr = fmt.Errorf("cannot read %q: %w", path, err)
 				continue
 			}
 
 			pausePid, err = strconv.Atoi(string(b[:n]))
-			if err == nil && unix.Kill(pausePid, 0) == nil {
-				foundProcess = true
-				lastErr = nil
-				break
+			if err != nil {
+				lastErr = err
+				continue
 			}
 		}
-	}
-	if !foundProcess && pausePidPath != "" {
-		return BecomeRootInUserNS(pausePidPath)
+
+		if pausePid > 0 && unix.Kill(pausePid, 0) == nil {
+			joined, pid, err := joinUserAndMountNS(uint(pausePid), pausePidPath)
+			if err == nil {
+				return joined, pid, nil
+			}
+			lastErr = err
+		}
 	}
 	if lastErr != nil {
 		return false, 0, lastErr
 	}
-
-	return joinUserAndMountNS(uint(pausePid), pausePidPath)
+	return false, 0, fmt.Errorf("could not find any running process: %w", unix.ESRCH)
 }
 
 // ReadMappingsProc parses and returns the ID mappings at the specified path.
@@ -466,7 +554,7 @@ func ReadMappingsProc(path string) ([]idtools.IDMap, error) {
 			if err == io.EOF {
 				return mappings, nil
 			}
-			return nil, errors.Wrapf(err, "cannot read line from %s", path)
+			return nil, fmt.Errorf("cannot read line from %s: %w", path, err)
 		}
 		if line == nil {
 			return mappings, nil
@@ -474,7 +562,7 @@ func ReadMappingsProc(path string) ([]idtools.IDMap, error) {
 
 		containerID, hostID, size := 0, 0, 0
 		if _, err := fmt.Sscanf(string(line), "%d %d %d", &containerID, &hostID, &size); err != nil {
-			return nil, errors.Wrapf(err, "cannot parse %s", string(line))
+			return nil, fmt.Errorf("cannot parse %s: %w", string(line), err)
 		}
 		mappings = append(mappings, idtools.IDMap{ContainerID: containerID, HostID: hostID, Size: size})
 	}
@@ -511,7 +599,7 @@ func ConfigurationMatches() (bool, error) {
 		return true, nil
 	}
 
-	uids, gids, err := GetConfiguredMappings()
+	uids, gids, err := GetConfiguredMappings(false)
 	if err != nil {
 		return false, err
 	}
