@@ -2,39 +2,18 @@ package integration
 
 import (
 	"fmt"
-	"os"
 	"sort"
 	"strings"
 
 	. "github.com/containers/podman/v4/test/utils"
 	"github.com/docker/go-units"
-	. "github.com/onsi/ginkgo"
+	. "github.com/onsi/ginkgo/v2"
 	. "github.com/onsi/gomega"
 	. "github.com/onsi/gomega/gexec"
 )
 
 var _ = Describe("Podman images", func() {
-	var (
-		tempdir    string
-		err        error
-		podmanTest *PodmanTestIntegration
-	)
 
-	BeforeEach(func() {
-		tempdir, err = CreateTempDirInTempDir()
-		if err != nil {
-			os.Exit(1)
-		}
-		podmanTest = PodmanTestCreate(tempdir)
-		podmanTest.Setup()
-	})
-
-	AfterEach(func() {
-		podmanTest.Cleanup()
-		f := CurrentGinkgoTestDescription()
-		processTestResult(f)
-
-	})
 	It("podman images", func() {
 		session := podmanTest.Podman([]string{"images"})
 		session.WaitWithDefaultTimeout()
@@ -213,7 +192,7 @@ WORKDIR /test
 		result := podmanTest.Podman([]string{"images", "-q", "-f", "dangling=true"})
 		result.WaitWithDefaultTimeout()
 		Expect(result).Should(Exit(0), "dangling image output: %q", result.OutputToString())
-		Expect(result.OutputToStringArray()).Should(HaveLen(0), "dangling image output: %q", result.OutputToString())
+		Expect(result.OutputToStringArray()).Should(BeEmpty(), "dangling image output: %q", result.OutputToString())
 	})
 
 	It("podman pull by digest and list --all", func() {
@@ -316,7 +295,7 @@ WORKDIR /test
 
 		actual := podmanTest.Podman([]string{"images", "--sort", "created", "-q"})
 		actual.WaitWithDefaultTimeout()
-		Expect(expected.Out).Should(Equal(actual.Out))
+		Expect(actual.Out).Should(Equal(expected.Out))
 	})
 
 	It("podman images --all flag", func() {
@@ -413,6 +392,16 @@ LABEL "com.example.vendor"="Example Vendor"
 		Expect(output).To(Equal("[]"))
 	})
 
+	It("podman images --filter label=with,comma", func() {
+		dockerfile := `FROM quay.io/libpod/alpine:latest
+`
+		podmanTest.BuildImageWithLabel(dockerfile, "foobar.com/before:latest", "false", "test=with,comma")
+		result := podmanTest.Podman([]string{"images", "--filter", "label=test=with,comma"})
+		result.WaitWithDefaultTimeout()
+		Expect(result).Should(Exit(0))
+		Expect(result.OutputToStringArray()).To(HaveLen(2))
+	})
+
 	It("podman images --filter readonly", func() {
 		dockerfile := `FROM quay.io/libpod/alpine:latest
 `
